<?php declare(strict_types=1);

namespace Everyware\Plugin\SettingsParameters;

use Infomaker\Everyware\Base\Models\SettingsParameter;
use Everyware\ProjectPlugin\Components\ComponentSettingsRepository;
use Everyware\ProjectPlugin\Components\SettingsProviders\CollectionDbProvider;
use Infomaker\Everyware\Support\Storage\CollectionDB;


class PluginSettingsParameters extends ComponentSettingsRepository
{
    private const SUBMIT_KEY = 'everyware-settings-parameters-submit';
    public const OPTION_NAME = 'everyware_settings_parameters';
    /**
     * @var \Infomaker\Everyware\Support\Storage\CollectionDB
     */
    private $collection;

    public function __construct()
    {
        if (is_admin()) {
            add_action( 'wp_ajax_delete_parameter', array( $this, 'ajaxDeleteParameter' ) );
            add_action( 'wp_ajax_add_parameters', array( $this, 'ajaxAddParameters' ) );
            add_action( 'wp_ajax_import_json', array( $this, 'ajaxJsonImport' ) );
        }

        parent::__construct(new CollectionDbProvider(new CollectionDB(static::OPTION_NAME), []));
    }
    /**
     * Ajax for merging jsonData into current settings parameters.
     *
     * @return void
     */
    public function ajaxJsonImport()
    {  
        if(isset($_POST['data']))
        {
            $importedParametersJson = stripslashes(html_entity_decode($_POST['data']));

            $importedParameters = json_decode($importedParametersJson,true);
           
            wp_send_json($this->mergeParameters($importedParameters));
        }
    }
    /**
     * Ajax for deleteing a parameter. Requires id to be present.
     *
     * @return void
     */
    public function ajaxDeleteParameter()
    {  
        if(isset($_POST['form']) && isset($_POST['data']))
        {
            parse_str($_POST['form'], $searcharray);
            $newParameters = $this->deleteParameter($searcharray['fields'], $_POST['data']);
          
            wp_send_json($this->updateParameters($newParameters));
        }
    }
    /**
     * Ajax for adding parameters
     *
     * @return void
     */
    public function ajaxAddParameters()
    {
        if(isset($_POST['form']))
        {
            parse_str($_POST['form'], $searcharray);
            wp_send_json($this->updateParameters($searcharray['fields']));
        }
    }
    /**
     * Merges saved and imported parameters and tries to save the result.
     *
     * @param array $importedParameters
     * @return bool
     */
    public function mergeParameters(array $importedParameters)
    {
        $this->collection = SettingsParameter::collection();
        $savedParameters = $this->collection->all()->flatten()->toArray();
        
        //Get all keys and values from the saved collection
        $array_keys = array_column($savedParameters, 'key');
        $array_values = array_column($savedParameters, 'value');
        //Combine keys and values array
        $savedCombined = array_combine($array_keys, $array_values);
        
        // Get all keys and values from importedParameters
        $array_keys = array_column($importedParameters, 'key');
        $array_values = array_column($importedParameters, 'value');
        //Combine keys and values array
        $importedCombined = array_combine($array_keys, $array_values);

        //Merge arrays
        $mergedParameters = array_merge($savedCombined, $importedCombined);

        //Get all values and keys into seperate arrays to create the correct structure for saving to collection
        $correctStructure = [
            'key' => array_keys($mergedParameters),
            'value' => array_values($mergedParameters)
        ];

        return $this->updateParameters($correctStructure);
    }

    /**
     * Deletes a item from the collection by getting its index from the passed key
     *
     * @param [type] $key
     * @return array
     */
    public function deleteParameter(array $parameters, $key): array
    {
        $id = array_search($key, array_values($parameters['key']));
        unset($parameters['key'][$id]);
        unset($parameters['value'][$id]);
        return $parameters;
    }

    /**
     * Iterates over all parameters and removes parameters with duplicate keys.
     *
     * @param array $parameters
     * @return array
     */
    public function deleteDuplicates(array $parameters): array
    {
        $duplicates = array();
        foreach(array_count_values($parameters['key']) as $value => $parameterKey) {
            if($parameterKey > 1) {
                $duplicates[] = $value;
            } 
        }
        foreach($duplicates as $duplicate) {
            $parameters = $this->deleteParameter($parameters, $duplicate);
        }
        return $parameters;
    }
    /**
     * Gets a new collection instance and updates and saves the parameters to the database.
     * This function handles updating, removing and adding new parameters.
     * @param array $parameters
     * @return bool
     */
    public function updateParameters(array $parameters): bool 
    {
        $this->collection = SettingsParameter::collection();
        return $this->collection->setCollection($this->getSubmittedParameters($parameters))->save();
    }
    /**
     * Get all parameters submitted by the form, both new and old and make a new collection
     *
     * @return array
     */
    public function getSubmittedParameters(array $submittedParameters): array
    {
        $parameters = $this->deleteDuplicates($submittedParameters);
        
        $validParameters = [];
        foreach ($parameters as $field => $values) {
            foreach ($values as $index => $value) {
                if ( ! isset($validParameters[$index])) {//Ny index
                    $validParameters[$index] = [];
                }

                $validParameters[$index][$field] = $value;
            }
        }
        return [
            'parameters' => $validParameters
        ];   
    }
}
